SET NOCOUNT ON;

IF EXISTS (SELECT * FROM tempdb.sys.tables WHERE tables.name LIKE '#view_metadata_before%')
BEGIN
	DROP TABLE #view_metadata_before;
END
CREATE TABLE #view_metadata_before
(	view_metadata_before_id INT NOT NULL IDENTITY(1,1) PRIMARY KEY clustered,
	view_object_id INT NOT NULL,
	view_schema_name SYSNAME NOT NULL,
	view_name SYSNAME NOT NULL,
	column_name SYSNAME NOT NULL,
	column_id INT NOT NULL,
	user_type_id INT NOT NULL,
	user_type_name SYSNAME NOT NULL,
	column_max_length SMALLINT NOT NULL,
	column_precision TINYINT NOT NULL,
	column_scale TINYINT NOT NULL,
	is_column_nullable BIT NOT NULL,
	is_schemabound_view BIT NOT NULL);
	
IF EXISTS (SELECT * FROM tempdb.sys.tables WHERE tables.name LIKE '#view_metadata_after%')
BEGIN
	DROP TABLE #view_metadata_after;
END
CREATE TABLE #view_metadata_after
(	view_metadata_after_id INT NOT NULL IDENTITY(1,1) PRIMARY KEY CLUSTERED,
	view_object_id INT NOT NULL,
	view_schema_name SYSNAME NOT NULL,
	view_name SYSNAME NOT NULL,
	column_name SYSNAME NOT NULL,
	column_id INT NOT NULL,
	user_type_id INT NOT NULL,
	user_type_name SYSNAME NOT NULL,
	column_max_length SMALLINT NOT NULL,
	column_precision TINYINT NOT NULL,
	column_scale TINYINT NOT NULL,
	is_column_nullable BIT NOT NULL,
	is_schemabound_view BIT NOT NULL);

INSERT INTO #view_metadata_before
	(view_object_id, view_schema_name, view_name, column_name, column_id, user_type_id, user_type_name, column_max_length, column_precision, column_scale, is_column_nullable, is_schemabound_view)
SELECT
	views.object_id AS view_object_id,
	schemas.name AS view_schema_name,
	views.name AS view_name,
	columns.name AS column_name,
	columns.column_id,
	types.user_type_id,
	types.name AS user_type_name,
	CASE WHEN types.name IN ('nchar', 'nvarchar') THEN columns.max_length / 2 ELSE columns.max_length END AS column_max_length,
	columns.precision AS column_precision,
	columns.scale AS column_scale,
	columns.is_nullable,
	OBJECTPROPERTY(views.object_id, 'IsSchemaBound') AS is_schemabound_view
FROM sys.views
INNER JOIN sys.columns
ON columns.object_id = views.object_id
INNER JOIN sys.types
ON types.user_type_id = columns.user_type_id
INNER JOIN sys.schemas
ON schemas.schema_id = views.schema_id
WHERE views.name NOT LIKE 'x[_]%'
ORDER BY views.object_id, columns.column_id;

IF EXISTS (SELECT * FROM tempdb.sys.tables WHERE tables.name LIKE '#distinct_view_list%')
BEGIN
	DROP TABLE #distinct_view_list;
END
CREATE TABLE #distinct_view_list
	(distinct_view_list_id INT NOT NULL IDENTITY(1,1) PRIMARY KEY CLUSTERED,
	 view_schema_name SYSNAME NOT NULL,
	 view_name SYSNAME NOT NULL);

INSERT INTO #distinct_view_list
	(view_schema_name, view_name)
SELECT DISTINCT
	view_metadata_before.view_schema_name,
	view_metadata_before.view_name
FROM #view_metadata_before view_metadata_before
WHERE view_metadata_before.is_schemabound_view = 0;

CREATE TABLE #view_ddl_exception_list
	(view_name SYSNAME NOT NULL PRIMARY KEY CLUSTERED);

DECLARE @sql_command NVARCHAR(MAX);
SELECT @sql_command = '';
SELECT
	@sql_command = @sql_command + '
	BEGIN TRY
		EXEC sp_refreshview N''' + CAST(distinct_view_list.view_schema_name AS VARCHAR(MAX)) + '.' + CAST(distinct_view_list.view_name AS VARCHAR(MAX)) + ''';
	END TRY
	BEGIN CATCH
		PRINT ''The following view is broken via DDL errors during sp_refreshview: ' + CAST(distinct_view_list.view_schema_name AS VARCHAR(MAX)) + '.' + CAST(distinct_view_list.view_name AS VARCHAR(MAX)) + ''';

		IF @@TRANCOUNT > 0
		BEGIN
			ROLLBACK;
		END

		INSERT INTO #view_ddl_exception_list
			(view_name)
		SELECT 
			''' + CAST(distinct_view_list.view_name AS VARCHAR(MAX)) + ''';
	END CATCH;'
FROM #distinct_view_list distinct_view_list;

EXEC sp_executesql @sql_command;

INSERT INTO #view_metadata_after
	(view_object_id, view_schema_name, view_name, column_name, column_id, user_type_id, user_type_name, column_max_length, column_precision, column_scale, is_column_nullable, is_schemabound_view)
SELECT
	views.object_id AS view_object_id,
	schemas.name AS view_schema_name,
	views.name AS view_name,
	columns.name AS column_name,
	columns.column_id,
	types.user_type_id,
	types.name AS user_type_name,
	CASE WHEN types.name IN ('nchar', 'nvarchar') THEN columns.max_length / 2 ELSE columns.max_length END AS column_max_length,
	columns.precision AS column_precision,
	columns.scale AS column_scale,
	columns.is_nullable,
	OBJECTPROPERTY(views.object_id, 'IsSchemaBound') AS is_schemabound_view
FROM sys.views
INNER JOIN sys.columns
ON columns.object_id = views.object_id
INNER JOIN sys.types
ON types.user_type_id = columns.user_type_id
INNER JOIN sys.schemas
ON schemas.schema_id = views.schema_id
WHERE views.name NOT LIKE 'x[_]%'
ORDER BY views.object_id, columns.column_id;

WITH CTE_before_except AS (
	SELECT
		view_object_id,
		view_schema_name,
		view_name,
		column_name,
		column_id,
		user_type_id,
		user_type_name,
		column_max_length,
		column_precision,
		column_scale,
		is_column_nullable,
		is_schemabound_view
	FROM #view_metadata_before
	EXCEPT
	SELECT
		view_object_id,
		view_schema_name,
		view_name,
		column_name,
		column_id,
		user_type_id,
		user_type_name,
		column_max_length,
		column_precision,
		column_scale,
		is_column_nullable,
		is_schemabound_view
	FROM #view_metadata_after),
CTE_after_except AS (
	SELECT
		view_object_id,
		view_schema_name,
		view_name,
		column_name,
		column_id,
		user_type_id,
		user_type_name,
		column_max_length,
		column_precision,
		column_scale,
		is_column_nullable,
		is_schemabound_view
	FROM #view_metadata_after
	EXCEPT
	SELECT
		view_object_id,
		view_schema_name,
		view_name,
		column_name,
		column_id,
		user_type_id,
		user_type_name,
		column_max_length,
		column_precision,
		column_scale,
		is_column_nullable,
		is_schemabound_view
	FROM #view_metadata_before)
SELECT
	CTE_before_except.view_object_id,
	CTE_before_except.column_id,
	CTE_before_except.view_schema_name AS view_schema_name_before,
	view_metadata_after.view_schema_name AS view_schema_name_after,
	CTE_before_except.view_name AS view_name_before,
	view_metadata_after.view_name AS view_name_after,
	CTE_before_except.column_name AS column_name_before,
	view_metadata_after.column_name AS column_name_after,
	CTE_before_except.user_type_id AS user_type_id_before,
	view_metadata_after.user_type_id AS user_type_id_after,
	CTE_before_except.user_type_name AS user_type_name_before,
	view_metadata_after.user_type_name AS user_type_name_after,
	CTE_before_except.column_max_length AS column_max_length_before,
	view_metadata_after.column_max_length AS column_max_length_after,
	CTE_before_except.column_precision AS column_precision_before,
	view_metadata_after.column_precision AS column_precision_after,
	CTE_before_except.column_scale AS column_scale_before,
	view_metadata_after.column_scale AS column_scale_after,
	CTE_before_except.is_column_nullable AS is_column_nullable_before,
	view_metadata_after.is_column_nullable AS is_column_nullable_after,
	CTE_before_except.is_schemabound_view
INTO #sp_refreshview_results
FROM CTE_before_except
LEFT JOIN #view_metadata_after view_metadata_after
ON view_metadata_after.view_object_id = CTE_before_except.view_object_id
AND view_metadata_after.column_id = CTE_before_except.column_id
UNION
SELECT
	view_metadata_before.view_object_id,
	view_metadata_before.column_id,
	view_metadata_before.view_schema_name AS view_schema_name_before,
	CTE_after_except.view_schema_name AS view_schema_name_after,
	view_metadata_before.view_name AS view_name_before,
	CTE_after_except.view_name AS view_name_after,
	view_metadata_before.column_name AS column_name_before,
	CTE_after_except.column_name AS column_name_after,
	view_metadata_before.user_type_id AS user_type_id_before,
	CTE_after_except.user_type_id AS user_type_id_after,
	view_metadata_before.user_type_name AS user_type_name_before,
	CTE_after_except.user_type_name AS user_type_name_after,
	view_metadata_before.column_max_length AS column_max_length_before,
	CTE_after_except.column_max_length AS column_max_length_after,
	view_metadata_before.column_precision AS column_precision_before,
	CTE_after_except.column_precision AS column_precision_after,
	view_metadata_before.column_scale AS column_scale_before,
	CTE_after_except.column_scale AS column_scale_after,
	view_metadata_before.is_column_nullable AS is_column_nullable_before,
	CTE_after_except.is_column_nullable AS is_column_nullable_after,
	CTE_after_except.is_schemabound_view
FROM CTE_after_except
LEFT JOIN #view_metadata_before view_metadata_before
ON view_metadata_before.view_object_id = CTE_after_except.view_object_id
AND view_metadata_before.column_id = CTE_after_except.column_id;

IF EXISTS (SELECT * FROM #sp_refreshview_results)
BEGIN
	SELECT
		*
	FROM #sp_refreshview_results;
END

DROP TABLE #view_metadata_before;
DROP TABLE #view_metadata_after;
DROP TABLE #distinct_view_list;
DROP TABLE #sp_refreshview_results;
DROP TABLE #view_ddl_exception_list;
GO



